#!/usr/bin/env python
# -*- mode: python -*- -*- coding: utf-8 -*-
#
#  rtm.py - rtm common module
#
#   Copyright (C) 2011 SAKODA, Sumiya <higebobo@gmail.com>
#

"""\
rtm common module
"""

__author__ = "Sumiya Sakoda"
__version__ = "$Revision$"
__date__ = "$Date: 2011/04/06 07:45:13 $"

import hashlib
import urllib

try:
    import json
except ImportError:
    try: 
        import simplejson as json
    except ImportError:
        import django.utils.simplejson as json

from var import API_KEY, SHARED_SECRET, TOKEN

FORMAT = 'json'
API_URL = 'http://api.rememberthemilk.com/services/rest/'
AUTH_URL = 'http://www.rememberthemilk.com/services/auth/'

class RTMError(Exception):
    '''RTM Error'''
    
    def __init__(self, value='RTM Error'):
        if value:
            self.value = value

    def __str__(self):
        return repr(self.value)

def _make_signature(**params):
    '''make api signature for rtm api'''
    
    # sort parameters by key and concat them with key and value
    sorted_params = [k + params[k] for k in sorted(params.keys())]
    
    # insert shared secret into joined parameters
    data = SHARED_SECRET + ''.join(sorted_params)
    
    # generate md5 hash
    signature = hashlib.md5(data).hexdigest()
    
    return signature

def _make_request_url(request_url, **params):
    '''make request url for rpm api'''
    
    return '%s?%s' % (request_url, urllib.urlencode(params))

def get_response(request_url=API_URL, **params):
    '''get response method'''
    
    # generate api signature
    api_sig = _make_signature(**params)

    # add api signature into parameters
    params['api_sig'] = api_sig

    # make request url for getting response
    _request_url = _make_request_url(request_url, **params)

    # get response
    response = urllib.urlopen(_request_url).read()

    return response

def get_task(task_filter=None):
    result = []
    
    # set parameters    
    params = {'api_key': API_KEY,
              'method': 'rtm.tasks.getList',
              'perms': 'read',
              'auth_token': TOKEN,
              'format': FORMAT}

    if task_filter:
        params['filter'] = task_filter

    # get response
    response = json.loads(get_response(**params))
    
    # parse json and return result
    if response['rsp']['stat'] == 'ok':
        tasks = response['rsp']['tasks']
        if not tasks.has_key('list'):
            return result
        tasks_list = tasks['list']
        if isinstance(tasks_list, dict):
            tasks_list = [tasks_list]
        for x in tasks_list:
            # skip no taskseries
            if not x.has_key('taskseries'):
                continue
            taskseries = x['taskseries']
            if isinstance(taskseries, dict):
                taskseries = [taskseries]
            for ts in taskseries:
                ts_id = ts['id']
                name = ts['name']
                tags = ts['tags']
                tag_list = []
                if tags:
                    tag_list = tags['tag']
                    if not isinstance(tag_list, list):
                        tag_list = [tag_list]
                ## for notes
                notes = ts['notes']
                if isinstance(notes, dict):
                    notes = [notes]
                note_list = []
                for n in notes:
                    for key, value in n.items():
                        if not isinstance(value, list):
                            value =  [value]
                        for v in value:
                            note_list.append(v['$t'])
                ## for task
                task = ts['task']
                if isinstance(task, list):
                    task = sorted(task, key=lambda x:x['added'],
                                  reverse=True)[0]
                completed = task['completed']
                due = task['due']
                priority = task['priority']
                task_id = task['id']
                result.append({'id': ts_id, 'name': name, 'due': due, 'priority': priority, 'notes': note_list, 'tags': tag_list})

        result = sorted(result, key=lambda x:x['due'])

    else:
        raise RTMError('Failed to get response')

    return result

def main():
    task_filter = 'status:incomplete AND dueWithin:"1 month of today"'
    for task in get_task(task_filter):
        print '*', task['due'], task['name'].encode('utf-8')
    
if __name__ == "__main__":
    main()
